function [z, eigenv, pred, confint_mean] = rsextreme(y, D, alfa, prefix)
% RSEXTREME - Fit a full quadratic model and describe it's stationary point. Version 0.1.30.
%
% The syntax is
% [z, eigenv, pred, confint_mean] = rsextreme(y, D, [alpha], [prefix])
%
% Arguments
% y is an N by 1 column vector of measured response data.
% D is an N by k design matrix with factor data.
% [alpha] is a voluntary parameter determining 1-alpha confidence. If omitted then alpha=0.05 is assumed.
% [prefix] is a voluntary string. If k=2 and [prefix] is passed then a pdf file is saved a contour plot with the location of the stationary point.
%
% Return values
% z is the location of the stationary point
% eigenv are the eigenvalues of the half Hessian B in the fitted model y=b0+x*b1+x*B*x'. All positive iff we have a local minimum, all negative iff we have a local maximum. If some eigenvalue is close to zero, then we have a ridge system.
% pred is the response predicted by the model at the stationary point
% confint_mean is a 1-alpha confidence interval for the prediction in the stationary point
% 
%
% References
% Myers, Montgomery & Anderson-Cook: Response Surface Methodology, 3rd ed., Wiley, 2009, Sections 6.3 and 6.5.2.


% Copyright (C) 2017 Mikael Kurula <mkurula@abo.fi>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
% FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, see <http://www.gnu.org/licenses/>.
%
% Author: Mikael Kurula <mkurula@abo.fi>, 2016


% Check input
if nargin < 2 || nargin > 4
    error('Try help rsextreme for instructions.') ;
end
if size(y,2) > 1,
    error('y has more than one column. Try help rsextreme for instructions.');
end
if nargin<3
  alfa=0.05;
end

% Extreme value analysis
[N,k]=size(D);
X=rsquad(D);
if cond(X'*X) > 1e10,
    warning('The normal equation is ill-conditioned. If the matrix D does not have linearly dependent columns or a column of ones, try standardizing the factor data.');
end
b=X\y;
b0=b(1);
b1=b(2:(k+1));
b(1:(k+1))=[];
F=zeros(k,k);
for j=0:(k-1)
  F=F+diag(b(1:(k-j)),j);
  b(1:(k-j))=[];
end
B=(F+F')/2;
eigenv=eig(B)';
z=-B^-1*b1/2;
z=z';

% Calculate confidence region for location of stationary point if k=2
if k==2
  [b,~,~,~,stats]=regress(y,X);
  a=size(X,2);
  Z=@(t) [zeros(k,1) eye(k) 2*diag(t) [t(2) t(1)]'];   % Last column correct only for k=2!
  d=@(t) Z(t)*b;                                       % Derivative of yhat at the point z. Z(t)*beta=zeros if t is the (unknown) theoretical stationary point.
  Vhat=@(t) stats(4)*Z(t)*inv(X'*X)*Z(t)';             % Standard error squared of d(t)         
  F=@(t) d(t)'*Vhat(t)^-1*d(t)/k;                      % F(k,N-a)-distributed test statistic  
  
  di=[z' D'];
  win=[min(di(1,:))-1 max(di(1,:))+1 min(di(2,:))-1 max(di(2,:))+1];
  steps=50;
  xx=linspace(win(1),win(2),steps);
  yy=linspace(win(3),win(4),steps);
  confreg=[];
  for j=1:steps
    for m=1:steps
      pt=[xx(j) yy(m)]';
      if F(pt) < finv(1-alfa,k,N-a)    % pt inside confidence region iff test statistic less than quantile
        confreg=[confreg pt];
      end
    end
  end
  
  rscontour(y,D,win)
  hold on
  plot(confreg(1,:),confreg(2,:),'b*','markersize',5,'linewidth',3)
  plot(di(1,2:end),di(2,2:end),'kx','linewidth',2)
  plot(z(1),z(2),'rd','linewidth',2)
  hold off
  title([num2str(1-alfa),'-confidence region for stationary point'])
  if nargin > 3 && size(prefix,1)>0,
    print([prefix,'-rsextreme'],'-dpdf')
  end
end

% Prediction at stationary point
x=rsquad(z);
[pred, confint_mean]=rspredict(y,X,x,alfa);

end
